<?php

/*
 * knitProPlus is based on knitPro - a web application for generating
 * knitting patterns
 * Copyright (C) 2004 - 2005  Cat Mazza & Eric St. Onge
 * 
 * Incorporates extensive enhancements for TheKnittingSite
 * <http://www.theknittingsite.com/>
 * Copyright (C) 2010 John Dablin
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Send questions or comments to penny@theknittingsite.com
 */

require_once("lib/knitPro.php");

// debugging!!!!
// In production environments, comment out the following line, or adjust
// the value to your requirements

//error_reporting(E_ALL | E_STRICT);
error_reporting(E_ALL);
// write error messages to a log file (change 'errors.log' to the
// name of your log file). This file must be writeable by the web server
ini_set('error_log', 'errors.log');
ini_set('log_errors', true);
 
// Set a default width and height of the page
// when it is in portrait mode.
// measurement is in point (72pt = 1in)
define("PAGE_WIDTH", 612);
define("PAGE_HEIGHT", 792);

// The minimum width of the page margin.
// This is per edge. 36pt means that there
// would be a 1/2 inch margin around
// the whole sheet
define("PAGE_MARGIN_HORIZONTAL", 36);
define("PAGE_MARGIN_VERTICAL", 36);

// The default number of boxes to use.
// Just in case none is provided.
define("DEFAULT_BOXES_HORIZONTAL", 48);
define("DEFAULT_BOXES_VERTICAL", 64);

// set the maximum number of boxes in the knitting pattern that
// should be displayed horizontally and vertically across the page.
// measurement is in point (72pt = 1in)
define("MAX_BOXES_HORIZONTAL", 48);
define("MAX_BOXES_VERTICAL", 64);

// The measurement in pt that the lines of the grid
// will extend past the grid
define("DEFAULT_GRID_EXTENSION_HORIZONTAL", 16);
define("DEFAULT_GRID_EXTENSION_VERTICAL", 16);


// set the square dimensions of for boxes in the knitting pattern.
// measurement is in point (72pt = 1in)
define("BOX_SIZE", 10);

// this variable defines the form element name for uploaded images.
// used in template.php.
define("UPLOAD_NAME", "userfile");

// if ENABLE_PROCESSED_FILE_STORAGE is enabled by setting its value to true,
// it will save the generated pattern to the directory specified below.
// make sure that knitPro has the correct permission to write files
// to the directory, otherwise the whole app will fail.
define("ENABLE_PROCESSED_FILE_STORAGE", "true");
define("PROCESSED_FILE_STORAGE_LOCATION", "result");

// just like processed file storage, except this saves
// the images people submit directly to the location.
define("ENABLE_UNPROCESSED_FILE_STORAGE", "false");
define("UNPROCESSED_FILE_STORAGE_LOCATION", "/home/eric/knitPro_files");

// this variable is a placeholder for an error message. 
// if no error has occurred, the value should be null.
$errMsg = null;
// this variable is a placeholder for the name of the generated PDF file.
// Until the file is generated its value should be null
$pdf_file = null;

// use a switch to determine which action to perform.
// there are really only two options:
//  1. display form for user to submit a pattern
//  2. send the generated pattern to the user.
// if an error occurs while generating the pattern, display
// the form page again with the error displayed.

if (isset($_POST['action']) && $_POST['action'] == 'generate')
{


    // if the user has uploaded a file, the internal PHP array _FILES
    // will store the information in an array indexed by the
    // form's NAME element. this is defined above in UPLOAD_NAME,
    // but the form in template.php should also use this name.
    
    // knitPro behaves differently depending on what type of image was submitted.
    // the MIME type of the image is also sent with the uploaded file.
    // this switch statement turns all the supported file formats into an
    // image resource in PHP. once PHP has an image resource, the file format is
    // not relevant, so the logic after the resource is created is common for all types.

        // initialise $img in case no image can be generated.
        $img = null;
        if ($_FILES[UPLOAD_NAME]["error"] != 0)
        {
	   // PHP has predefined error codes for image uploads.
	   // full docs at: http://us3.php.net/manual/en/features.file-upload.errors.php
		if ($_FILES[UPLOAD_NAME]["error"] == 2
			|| $_FILES[UPLOAD_NAME]["error"] == 1) {
			$errMsg = "ERROR: The file you uploaded was too large. Please choose a smaller file and try again.";
		}
		else if ($_FILES[UPLOAD_NAME]["error"] == 3) {
			$errMsg = "ERROR: The file was not uploaded successfully. Please try again.";
                }
                else if ($_FILES[UPLOAD_NAME]["error"] == 4) {
                        $errMsg = "ERROR: No image was selected. Please choose an image to process and try again.";
		}
                else
                        $errMsg = "ERROR: File upload failed.";
        }
        else
        {
            switch($_FILES[UPLOAD_NAME]["type"]) {

                // GIF file type
                    case "image/gif":
                            $img = ImageCreateFromGIF($_FILES[UPLOAD_NAME]["tmp_name"]);
                            break;
                            
            // JPEG and assorted variants
                    case "image/pjpeg": // Internet Explorer sends pjpegs for some reason.
                    case "image/jpg":
                    case "image/jpeg":
                    case "image/jpe":
                // the same PHP function is used for all of the JPEG file types.
                            $img = ImageCreateFromJPEG($_FILES[UPLOAD_NAME]["tmp_name"]);
                            break;
                            
                    case "image/png":
                            $img = ImageCreateFromPNG($_FILES[UPLOAD_NAME]["tmp_name"]);
                            break;

            // if no image was submitted, display a more specific error message.		
                    case "":
                            $errMsg = "ERROR: No image was selected. Please choose an image to process and try again.";
                            break;
                            
                    // if we get something we don't expect, end the program and set an error.
                    // knitPro can use whatever file types PHP supports: http://us2.php.net/imagecreate
                    // GIF, JPEG, and PNG are common formats that you should expect to see.
                    default:
                            $errMsg = "ERROR: Unsupported file format (" . $_FILES[UPLOAD_NAME]["type"] . "). Please select a GIF, JPG, or PNG image and try again.";
                            break;
            }
	}
			
    // if PHP was able to make an image resource, continue with the app.
	if (!$img) {
	
            if (is_null($errMsg)) {
                    $errMsg = "ERROR: Could not load image.";
            }
	}

        if ($_POST['stitch_preset'] == '4')
        {
            $_POST['rows'] = trim($_POST['rows']);
            $_POST['stitches'] = trim($_POST['stitches']);
            if (!preg_match("/^[0-9]{1,3}$/", $_POST['stitches']))
                $errMsg = "ERROR: tension stitches is not a valid number.";
            elseif (!preg_match("/^[0-9]{1,3}$/", $_POST['rows']))
                $errMsg = "ERROR: tension rows is not a valid number.";
        }
	
	// if no error message has been set, generate the PDF.
	if (is_null($errMsg)) {
	
	   $boxes_horizontal = DEFAULT_BOXES_HORIZONTAL;
	   $boxes_vertical = DEFAULT_BOXES_VERTICAL;

	   // several "presets" are defined in the submission form.
	   // change the presets back to their knitPro settings here.
	   // (case 1 is skipped since it should already be set to those values)
	   switch($_POST["grid_preset"]) {
        case '1':
           $boxes_horizontal = 21;
	   $boxes_vertical = 28;
           break;

        case '2':
           $boxes_horizontal = 30;
	   $boxes_vertical = 40;
           break;

        case '3':
           break;       // default - values are already set above
	
        case '4':
            $boxes_horizontal = 96;
            $boxes_vertical = 128;
            break;
            
        case '5':
            $boxes_horizontal = 120;
            $boxes_vertical = 160;
            break;
        
	   }
	   
	   $box_horizontal_scale = "1";
	   $box_vertical_scale = "1";
	   
	   // do the same here for stich aspect ratio presets.
	   
	   switch ($_POST["stitch_preset"]) {

        case '2':
            $box_horizontal_scale = "5";
            $box_vertical_scale = "7";
            break;
	   
        case '3':
            $box_horizontal_scale = "5";
            $box_vertical_scale = "4";
            break;
	   
        case '4':
            // 'rows' and 'stitches' are the number of stitches in
            // a constant length, but what we need is the ratio of
            // height to width, which is the inverse of the numbers
            // entered. It can be shown that this can be reduced to
            // simply swapping the two numbers
            $box_horizontal_scale = $_POST["rows"];
            $box_vertical_scale = $_POST["stitches"];
            break;
	   }

        // generate the PDF
            $pdf_file = generateKnitProPDF($img, $boxes_horizontal,
                        $boxes_vertical, $box_horizontal_scale,
                        $box_vertical_scale);
	}
}

// the UI is all defined in the file template.php.
// if $pdf_file is null, display the form
// if an error has been set (errMsg is not null) display the form with
// an error message
// otherwise display "success" and a link to download the file
include("template.php");

?>
