<?

/*
 * knitProPlus is based on knitPro - a web application for generating
 * knitting patterns
 * Copyright (C) 2004 - 2005  Cat Mazza & Eric St. Onge
 * 
 * Incorporates extensive enhancements for TheKnittingSite
 * <http://www.theknittingsite.com/>
 * Copyright (C) 2010 John Dablin
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Send questions or comments to penny@theknittingsite.com
 */


// load a the rospdf class. knitPro uses this class
// to generate the kniting pattern as a pdf.

// rospdf requires that PHP has the GD library installed. 
include('./lib/rospdf009e/class.pdf.php');

// unique_name creates a unique name for a directory to store the resulting file
function unique_name()
{
    // create a unique string from the client's IP address and port, then
    // append a random number
    $name = uniqid($_SERVER['REMOTE_ADDR'] . ':' . $_SERVER['REMOTE_PORT'], true);
    $name .= '-' . mt_rand(1,1000);
    // now use md5 to create a unique hex string
    return md5($name);
}

// clean_up deletes files from the file storage location that are over 24 hours
// old
// $dir is the full pathname of the directory containing the files

function clean_up($dir)
{
    if ($d = opendir($dir))
    {
        // read the name of each file in the directory
        while (false !== ($filename = readdir($d)))
        {
            // Ignore '.' and '..' !!!
            if ($filename != '.' && $filename != '..')
            {
                // create a full pathname
                $fullname = $dir . '/' . $filename;
                if ((time() - filemtime($fullname)) > 24*60*60)
                {
                    // the file is more than a day old
                    // if it's a file, delete it
                    if (is_file($fullname))
                        unlink($fullname);
                    else
                    {
                        // otherwise recursively clean up a subdirectory
                        clean_up($fullname);
                        rmdir($fullname);
                    }
                }
            }
        }
    }
}

// generateKnitProPDF() is the function that does all of the work for
// the application.

function generateKnitProPDF($img, 
    $boxes_horizontal = "48", $boxes_vertical = "64", 
    $box_horizontal_scale = "5", $box_vertical_scale = "7",
    $grid_extension_horizontal = "16", $grid_extension_vertical = "16",
    $margin_horizontal = "36", $margin_vertical = "36",
    $grid_line_width = "1",
    $page_width = "612", $page_height = "792",
    $print_debug = false
        )
{
    // delete old files
    if (ENABLE_PROCESSED_FILE_STORAGE == "true")
        clean_up(getcwd() . '/' . PROCESSED_FILE_STORAGE_LOCATION);

    if ($print_debug) {
        echo "<p>boxes_horizontal: " . $boxes_horizontal
             . "<p>boxes_vertical: " . $boxes_vertical;
    }
	
	// Get the width of the image
	$width = ImagesX($img);

	// Get the height of the image
	$height = ImagesY($img);

    // initialize the "scale factor" for the image.
    // if the image is too big to be displayed in the dimensions
    // required, it will be scaled down to fit.
    // knitPro will not ever scale an image up to fit, since that
    // could result in distorted images.
	$scale = 1;

    // figure out if the aspect ratio of the image.
    // we change the dimensions of the page accordingly
    // if the image is in landscape or portrait aspect ratios.

    // if the image is more wide than tall, the pattern should
    // be generated using landscape aspect ratio
	if ($width > $height) {
	
        // get the preset width and height of the page (in point)
        // since the defaults are for portrait mode
        // and this image is a landscape, swap the parameters.
		$PAGE_WIDTH = PAGE_HEIGHT;
		$PAGE_HEIGHT = PAGE_WIDTH;	
		
		// if there are more vertical pixels in the image
		// than the number of boxes we have available,
		// figure out how much we have to scale the image down.
		if (($width * $scale) >= $boxes_vertical) 
			$scale = ($boxes_vertical / $width);
		
		// we still might have to scale the image in the other
		// direction, too.
		if (($height * $scale) >= $boxes_horizontal)
			$scale = $boxes_horizontal / $height;
		
			
	}
	else {
	
	
	    // do the same as above, but for images that are in
	    // either square or portrait aspect ratios.

		$PAGE_WIDTH = PAGE_WIDTH; // 11in
		$PAGE_HEIGHT = PAGE_HEIGHT; // 8.5in	

        
		if (($height * $scale) >= $boxes_vertical) 
			$scale = $boxes_vertical / $height;

		if (($width * $scale) >= $boxes_horizontal)
			$scale = $boxes_horizontal / $width;
        
	}
	
	// the number of boxes must be a whole number,
	// so figure out the number of boxes to use
	// by scaling down.
	$px_vt_scale = ($box_vertical_scale >= $box_horizontal_scale
	                    ? 1
	                    : ($box_vertical_scale / $box_horizontal_scale)
	                    );

    $px_hz_scale = ($box_horizontal_scale >= $box_vertical_scale
                        ? 1
                        : ($box_horizontal_scale / $box_vertical_scale)
                        );	
                        
	
	$horizontalPixels = floor($width * $scale * $px_vt_scale);
	$verticalPixels = floor($height * $scale * $px_hz_scale);
	
	if ($print_debug) {
	   echo "<p>px_hz_scale: " . $px_hz_scale . "<p>px_vt_scale: " . $px_vt_scale;
    }
		
	// convert the image to RGB mode so that we can reference
	// the colors correctly for all color types.
	$newImg = ImageCreateTrueColor($horizontalPixels, $verticalPixels);

    // define the first color to be black. This turns all transparent pixels black.
    // transparency could probably be handled better...
	ImageColorTransparent($img, 0);

	/*
	//@todo
	$white = ImageColorAllocate($newImg, 1, 1, 1);
	ImageFill($newImg, $horizontalPixels, $verticalPixels, $white);
	*/
 
    // copy the old image into the new RGB image reference
    // and resize the image as necessary
	ImageCopyResampled($newImg, $img,
							0, 0,
							0, 0,
							$horizontalPixels, $verticalPixels,
							$width, $height);

    // since the image has now been resized, reset the original image width and height
    // to the new image dimensions
	$width = $horizontalPixels;
	$height = $verticalPixels;
	
	if ($print_debug) {
	   echo "<p>width: $width\n<p>height: $height\n";
	}

	$img = $newImg;
	
	//$squareSize = BOX_SIZE;
	
    // get the maximum possible dimensions for the image
	$maximum_point_horizontal = $PAGE_WIDTH;
	$maximum_point_vertical = $PAGE_HEIGHT;
	
	if ($print_debug) {
    	echo "<p>maximum_point_horizontal: " . $maximum_point_horizontal 
    	   . "<p>maximum_point_vertical: " . $maximum_point_vertical;
	}

    // get the minimum possible dimensions for the image --
    // this includes all the borders and margins that we know
    // we have to use.
	$minimum_point_horizontal = (2 * $margin_horizontal) 
	   + (2 * $grid_extension_horizontal);
	   
	$minimum_point_vertical = (2 * $margin_vertical) 
	   + (2 * $grid_extension_vertical);
	
	
	if ($print_debug) {
    	echo "<p>minimum_point_horizontal: " . $minimum_point_horizontal 
    	   . "<p>minimum_point_vertical: " . $minimum_point_vertical;
	}

    
	$box_hz_size = 1;
	$box_vt_size = 1;
	
	// figure out much space is left for the actual processed image.
    $pt_available_vt = $maximum_point_vertical - $minimum_point_vertical;
    $pt_available_hz = $maximum_point_horizontal - $minimum_point_horizontal;
    	
   	if ($print_debug) {
   	   echo "<p>pt_available_hz: " . $pt_available_hz;
       echo "<p>pt_available_vt: " . $pt_available_vt;
    }
    
    
    // figure out if the image will fit in the grid we have,
    // and scale down the image if necessary.
	if ($px_vt_scale > $px_hz_scale) {
    	
    	$box_vt_size = ($pt_available_vt / $height);
    	$box_hz_size = $box_vt_size * $px_hz_scale;

    	// check to see if the scale is still too large...
    	if ( ($box_hz_size * $width) > $pt_available_hz ) {
            $scaleMore = ( $pt_available_hz / ($box_hz_size * $width) );            
            if ($print_debug) {
                echo "<p>scaling more: $scaleMore\n";
            }
            $box_vt_size *= $scaleMore;
            $box_hz_size *= $scaleMore;    	
    	}

	}
	else {

    	$box_hz_size = ($pt_available_hz / $width);
    	$box_vt_size = $box_hz_size * $px_vt_scale;

    	// check to see if the scale is still too large...
    	if ( ($box_vt_size * $height) > $pt_available_vt ) {
            $scaleMore = ( $pt_available_vt / ($box_vt_size * $height) );            
            if ($print_debug) {
                echo "<p>scaling more: $scaleMore\n";
            }
            $box_vt_size *= $scaleMore;
            $box_hz_size *= $scaleMore;    	
    	}
	}
		
	if ($print_debug) {
    	echo "<p>box_hz_size: " . $box_hz_size . "<p>box_vt_size: " . $box_vt_size;
	}
	
    // figure out how big the grid will be so that we can place it on
    // the page correctly.
	$gridHorizontalSize = $box_hz_size * $horizontalPixels;
	$gridVerticalSize = $box_vt_size * $verticalPixels;
	
	if ($print_debug) {
    	echo "<p>gridHorizontalSize: " . $gridHorizontalSize . "<p>gridVerticalSize: " . $gridVerticalSize;
	}
	
	
	// determine the coordinates for placement of the grid.
	$baseX = ($PAGE_WIDTH - $gridHorizontalSize) / 2;
	$baseY = ($PAGE_HEIGHT - $gridVerticalSize) / 2;

    // create a new PDF object so that we can start drawing the grid
	$pdf = new Cpdf(array(0,0, $PAGE_WIDTH, $PAGE_HEIGHT));
	
	// select a default font of Helvetica
	$pdf->selectFont('lib/rospdf009e/fonts/Helvetica');
	
	// draw a URL to theknittingsite.com in the lower left corner of the page.
	$pdf->addText(16, 16, 10, "http://www.theknittingsite.com/");

    // use a standard line style 
	$pdf->setLineStyle(0);
	
	// this is where the bulk of the work occurs.

	// go through each pixel of the image and draw an element in the grid for it.
	for ($x = 0; $x < $width; $x++) {
		for ($y = 0; $y < $height; $y++) {
		
		    // get the color at this pixel in the resampled image
			$colorIndex = ImageColorAt($img, $x, $height - 1 - $y);
			$color = ImageColorsForIndex($img, $colorIndex);

            // change back to indexed color mode (since that's all the PDF generator knows)
			$pdf->setColor($color["red"] / 255, $color["green"] / 255, $color["blue"] / 255);
			
			// generate the rectangle "grid" box
			$pdf->filledRectangle(	$baseX + ($box_hz_size * $x), 
									$baseY + ($box_vt_size * $y),
									$box_hz_size, $box_vt_size);
		}
	}

	//$pdf->setColor(0, 0, 0);
	//$pdf->setColor( (51/255), (204/255), (255/255) );
	
    // now draw lines in the image to show the grid
	// 1pt straight line  in a light blue.
	$pdf->setLineStyle(1);
	$pdf->setStrokeColor(0.2, 0.8, 1.0);
			
	// for horizontal pixels, draw vertical lines
	for ($t = 0; $t <= $horizontalPixels; $t++) {
		$pdf->line( 	$baseX + ($t * $box_hz_size),	
							$baseY - $grid_extension_vertical,
						$baseX + ($t * $box_hz_size),	
							$baseY + $gridVerticalSize + $grid_extension_vertical);
							
		/*if ( ($t + 1) % 2 == 0) {
			$pdf->addText(, , 8, "<b>" . ($t + 1) . "</b>");
		}*/
	}

	// for vertical pixels, draw horizontal lines
	for ($t = 0; $t <= $verticalPixels; $t++) {
		$pdf->line( 	$baseX - $grid_extension_horizontal, 						
							$baseY + ($t * $box_vt_size),
						$baseX + $gridHorizontalSize + $grid_extension_horizontal, 	
							$baseY + ($t * $box_vt_size));
	}
	
	// draw line numbers for every other grid line
	
	// change font to black
	$pdf->setColor(0, 0, 0);
    
	// horizontal grid numbers
    $hNumPosition = $baseX - $grid_extension_vertical + 2;
    for ($i = 1; $i < $verticalPixels; $i += 2) {
        $vNumPosition = $baseY + ($i * $box_vt_size) + 2;
        $pdf->addText($hNumPosition, $vNumPosition, 6, "<b>" . ($i + 1). "</b>");
    }
    
	// vertical grid numbers
    $vNumPosition = $baseY - $grid_extension_horizontal + 2;
    for ($i = 1; $i < $horizontalPixels; $i += 2) {
        $hNumPosition = $baseX + ($i * $box_hz_size) + 2;
        $pdf->addText($hNumPosition, $vNumPosition, 6, "<b>" . ($i + 1). "</b>");
    }
	
	// if the option to save unprocessed files (originals) is enabled,
    // do that here.
    if (ENABLE_UNPROCESSED_FILE_STORAGE == "true") {
        // make up a name for the image
        $unprocessed_file_name = UNPROCESSED_FILE_STORAGE_LOCATION 
            . date("Y_m_d_H_i_s") . "." . $_FILES[UPLOAD_NAME]["name"];
            
        // move the file into place.
        if (!move_uploaded_file($FILES[UPLOAD_NAME]["tmp_name"], $unprocessed_file_name)) {
            echo("Could not write unprocessed file ($unprocessed_file_name)");
            exit();
        }
    }


	// if the option to save generated PDFs to disk has been
	// enabled, do that now.
	if (ENABLE_PROCESSED_FILE_STORAGE == "true") {
	
	   // get the PDF file contents.
	   $file_contents = $pdf->output();
	   
	   // make up a file name for this file.
	   // e.g., /home/eric/processed_knitPro_files/2005_11_28_11_00_31.myfile.gif.pdf
	   // this should always be unique.
           // get the base part of the uploaded filename, stripping off any suffix
           $input_file = $_FILES[UPLOAD_NAME]['name'];
           $input_file = basename($input_file, '.' . pathinfo($input_file, PATHINFO_EXTENSION));
	   $processed_file_name = $input_file . ".pdf";
           $processed_file_path = PROCESSED_FILE_STORAGE_LOCATION . '/' . unique_name();
           // create the unique directory to hold the file (ignore errors, if
           // the directory isn't created, the file open will fail)
           $abspath = getcwd() . '/' . $processed_file_path;
           if (!file_exists($abspath))
               mkdir($abspath, 0775, true);
	   // try to get a file handle for the new file.
           if (!$handle = fopen($abspath . '/' . $processed_file_name, "w")) {
              echo "Could not open new processed file ($processed_file_path . '/' . $processed_file_name)";
              exit();
            }
            else {
                // write the file to disk.
                if (fwrite($handle, $file_contents) === FALSE) {
                    echo "Could not write processed file ($processed_file_path . '/' . $processed_file_name)";
                    exit();
            }
            // close the file handle.
            fclose($handle);

            // return the relative path to the file to the caller
            return $processed_file_path . '/' . $processed_file_name;
        }
    }
    else
    {
    // send the generated pdf to the user's browser
	$pdf->stream();
        return true;
    }
}

?>
